--
-- AmazoneUX5200
-- Specialization for AmazoneUX5200
--
-- @author Felix "Outlaw" Sorge
-- @date  19/06/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
-- @edit  gotchTOM
-- @date  13/12/10

AmazoneUX5200 = {};

AmazoneUX5200.SPRAYTYPE_UNKNOWN = 0;
AmazoneUX5200.NUM_SPRAYTYPES = 0;
  
AmazoneUX5200.sprayTypes = {};
AmazoneUX5200.sprayTypeIndexToDesc = {};
  
AmazoneUX5200.sprayTypeToFillType = {};
AmazoneUX5200.fillTypeToSprayType = {};

function AmazoneUX5200.registerSprayType(name, pricePerLiter, litersPerSqmPerSecond, hudOverlayFilename)
        local key = "SPRAYTYPE_"..string.upper(name);
        if AmazoneUX5200[key] == nil then
            AmazoneUX5200.NUM_SPRAYTYPES = AmazoneUX5200.NUM_SPRAYTYPES+1;
            AmazoneUX5200[key] = AmazoneUX5200.NUM_SPRAYTYPES;
    
            local desc = {name = name, index = AmazoneUX5200.NUM_SPRAYTYPES};
            desc.pricePerLiter = pricePerLiter;
            desc.litersPerSqmPerSecond = litersPerSqmPerSecond;
            desc.hudOverlayFilename = hudOverlayFilename;
    
    
            AmazoneUX5200.sprayTypes[name] = desc;
            AmazoneUX5200.sprayTypeIndexToDesc[AmazoneUX5200.NUM_SPRAYTYPES] = desc;
    
            local fillType = Fillable.registerFillType(name)
            AmazoneUX5200.sprayTypeToFillType[AmazoneUX5200.NUM_SPRAYTYPES] = fillType;
            AmazoneUX5200.fillTypeToSprayType[fillType] = AmazoneUX5200.NUM_SPRAYTYPES;
        end;
    end;
	
	AmazoneUX5200.registerSprayType("fertilizer", 0.3, 0.5, "");

function AmazoneUX5200.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function AmazoneUX5200:load(xmlFile)

	assert(self.setIsTurnedOn == nil, "Sprayer needs to be the first specialization which implements setIsTurnedOn");
	self.setIsTurnedOn = AmazoneUX5200.setIsTurnedOn;
    self.getIsTurnedOnAllowed = AmazoneUX5200.getIsTurnedOnAllowed;
    
    assert(self.setIsSprayerFilling == nil, "Sprayer needs to be the first specialization which implements setIsSprayerFilling");
    self.setIsSprayerFilling = AmazoneUX5200.setIsSprayerFilling;
    self.addSprayerFillTrigger = AmazoneUX5200.addSprayerFillTrigger;
    self.removeSprayerFillTrigger = AmazoneUX5200.removeSprayerFillTrigger;
	self.fillLitersPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillLitersPerSecond"), 500);
	self.isSprayerFilling = false;

	self.sprayLitersPerSecond = {};
        local i=0;
        while true do
            local key = string.format("vehicle.sprayUsages.sprayUsage(%d)", i);
            if not hasXMLProperty(xmlFile, key) then
                break;
            end;
            local fillType = getXMLString(xmlFile, key.. "#fillType");
            local litersPerSecond = getXMLFloat(xmlFile, key.. "#litersPerSecond");
            if fillType ~= nil and litersPerSecond ~= nil then
                local fillTypeInt = Fillable.fillTypeNameToInt[fillType];
                if fillTypeInt ~= nil then
                    self.sprayLitersPerSecond[fillTypeInt] = litersPerSecond;
                    if self.defaultSprayLitersPerSecond == nil then
                        self.defaultSprayLitersPerSecond = litersPerSecond;
                    end;
                else
                    print("Warning: Invalid spray usage fill type '"..fillType.."' in '" .. self.configFileName.. "'");
                end;
            end;
            i = i+1;
        end;
        if self.defaultSprayLitersPerSecond == nil then
            print("Warning: No spray usage specified for '" .. self.configFileName.. "'. This sprayer will not use any spray.");
            self.defaultSprayLitersPerSecond = 0;
        end;
		
	self.sprayValves = {};    
			if self.isClient then
				local psFile = getXMLString(xmlFile, "vehicle.sprayParticleSystem#file");
				if psFile ~= nil then					
					local i=0;
					while true do
						local baseName = string.format("vehicle.sprayValves.sprayValve(%d)", i);
						local node = getXMLString(xmlFile, baseName.. "#index");
						if node == nil then
							break;
						end;
						node = Utils.indexToObject(self.components, node);
						if node ~= nil then
							local sprayValve = {};
							sprayValve.particleSystems = {};
							Utils.loadParticleSystem(xmlFile, sprayValve.particleSystems, "vehicle.sprayParticleSystem", node, false, nil, self.baseDirectory);
							table.insert(self.sprayValves, sprayValve);
						end;
					i = i+1;
					end;					
				end;  
				local spraySound = getXMLString(xmlFile, "vehicle.spraySound#file");
				if spraySound ~= nil and spraySound ~= "" then
					spraySound = Utils.getFilename(spraySound, self.baseDirectory);
					self.spraySound = createSample("spraySound");
					self.spraySoundEnabled = false;
					loadSample(self.spraySound, spraySound, false);
					self.spraySoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#pitchOffset"), 1);
					self.spraySoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#volume"), 1);
				end;
			end;
	self.exSprayValves = {};		
			if self.isClient then
				local psFile = getXMLString(xmlFile, "vehicle.sprayParticleSystem#file");
				if psFile ~= nil then
					local i=0;
					while true do
						local baseName = string.format("vehicle.exSprayValves.exSprayValve(%d)", i);
						local node = getXMLString(xmlFile, baseName.. "#index");
						if node == nil then
							break;
						end;
						node = Utils.indexToObject(self.components, node);
						if node ~= nil then
							local exSprayValve = {};
							exSprayValve.particleSystems = {};
							Utils.loadParticleSystem(xmlFile, exSprayValve.particleSystems, "vehicle.sprayParticleSystem", node, false, nil, self.baseDirectory);
							table.insert(self.exSprayValves, exSprayValve);
						end;
					i = i+1;
					end;					
				end;  
			end;	
	
   
	self.sprayerFillTriggers = {};
	self.sprayerFillActivatable = SprayerFillActivatable:new(self);   
	self.isTurnedOn = false;
	self.speedViolationMaxTime = 1000;
	self.speedViolationTimer = self.speedViolationMaxTime;	
	self.connectCollisions = SpecializationUtil.callSpecializationsFunction("connectCollisions");
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
	self.Go = {}; 
	self.Done = {};
	self.charId = {};
	self.clipIndex = {};
	self.CheckDone = {};
	self.moveColli = {}; 
    self.animParts = {};
	self.collisionArm = {};
	self.startpoint = {};
	self.loopCheck = {};
	self.Speed = {};
	local count = getXMLInt(xmlFile, "vehicle.animParts#count");
	local part = self.animParts;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.animParts.part".."%d", i);
	    local nameR = getXMLString(xmlFile, partname.."#name");
		self.charId[nameR] = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#rootNode"));
		self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
		self.CheckDone[nameR] = false;
		self.moveColli[nameR] = getXMLString(xmlFile, partname.."#moveColli");
		self.startpoint[nameR] = Utils.getNoNil(getXMLFloat(xmlFile, partname.."#startpoint"),0.0);
		if self.moveColli[nameR] ~= nil then
			local Collision = {};
			Collision.collision = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collision"));
			Collision.collisionAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collsionAttacher"));
			Collision.armAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.. "#armAttacher"));
			Collision.index = 0;
			self.collisionArm[nameR] = Collision;
			self:connectCollisions(nameR);
		end; 
		local charId = getAnimCharacterSet(self.charId[nameR]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[nameR]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackTime(charId, 0, self.startpoint[nameR]);
		enableAnimTrack(charId, 0);
		disableAnimTrack(charId, 0);
		self.loopCheck[nameR] = false;
		self.Speed[nameR] = 1.0;
	end; 
	local hydraulicSound = getXMLString(xmlFile, "vehicle.hydraulicSound#file");
    if hydraulicSound ~= nil and hydraulicSound ~= "" then
        hydraulicSound = Utils.getFilename(hydraulicSound, self.baseDirectory);
        self.hydraulicSound = createSample("hydraulicSound");
        self.hydraulicSoundEnabled = false;
        loadSample(self.hydraulicSound, hydraulicSound, false);
        self.hydraulicSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hydraulicSound#pitchOffset"), 1);
        self.hydraulicSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hydraulicSound#volume"), 1);
    end;
	local numWorkLights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workLights#count"), 0);
    self.workLights = {};
    for i=1, numWorkLights do
        local worklightnamei = string.format("vehicle.workLights.workLight%d", i);
        self.workLights[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, worklightnamei .. "#index"));
        setVisibility(self.workLights[i], false);
    end;
	self.workLightsActive = false;
	local nummarkAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.markAreas#count"), 0);
    self.markAreas = {}
    for i=1, nummarkAreas do
        self.markAreas[i] = {};
        local areanamei = string.format("vehicle.markAreas.markAreas%d", i);
        self.markAreas[i].point = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#pointIndex"));
        self.markAreas[i].width = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#width"), 0.5)
        self.markAreas[i].height = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#height"), 0.25)
    end;
	self.groundCheck = {};
	self.groundCheck.left = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.groundCheck#left"));
	self.groundCheck.right = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.groundCheck#right"));	
	--self.literPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.literPerSecond#value"), 1);
	self.UX5200helpPanel = false;
	setMass(self.rootNode, 1.5);
end;

function AmazoneUX5200:delete()
	g_currentMission:removeActivatableObject(self.sprayerFillActivatable);   
    for k,sprayValve in pairs(self.sprayValves) do
        Utils.deleteParticleSystem(sprayValve.particleSystems);
    end;
	for l,exSprayValve in pairs(self.exSprayValves) do
        Utils.deleteParticleSystem(exSprayValve.particleSystems);
    end;   
    if self.spraySound ~= nil then
        delete(self.spraySound);
    end;
	if self.hydraulicSound ~= nil then
        delete(self.hydraulicSound);
    end;
	for k, worklight in pairs(self.workLights) do
		setVisibility(worklight, false);
	end;
	self.workLightsActive = false;
end;

function AmazoneUX5200:readStream(streamId, connection)
       local turnedOn = streamReadBool(streamId);
       local isSprayerFilling = streamReadBool(streamId);
       self:setIsTurnedOn(turnedOn, true);
       self:setIsSprayerFilling(isSprayerFilling, true);
end;
   
function AmazoneUX5200:writeStream(streamId, connection)
       streamWriteBool(streamId, self.isTurnedOn);
       streamWriteBool(streamId, self.isSprayerFilling);
end;
   
 function AmazoneUX5200:readUpdateStream(streamId, timestamp, connection)
 end;
   
function AmazoneUX5200:writeUpdateStream(streamId, connection, dirtyMask)
end;
   
function AmazoneUX5200:mouseEvent(posX, posY, isDown, isUp, button)
end;
   
function AmazoneUX5200:keyEvent(unicode, sym, modifier, isDown)
end;

function AmazoneUX5200:update(dt)
	setMass(self.rootNode, 1.5);
	if self.isClient then
		if self:getIsActiveForInput() then	
			if self.attacherVehicle then
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_HLPANL) then
					self.UX5200helpPanel = not self.UX5200helpPanel;
				end;
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_TRSPRT) and not self.isTurnedOn and not self.levelCheck then
					self.Speed.trsp = 1.0; 
					self.Speed.ex = 1.0; 
					self.Go.trsp = not self.Go.trsp; 
					self.Done.trsp = true;  
					self.Go.hubmast = false; 
					self.Done.hubmast = true;
					if self.exSave then
						self.Go.ex = not self.Go.ex;
						self.Done.ex = true;
					end; 
				end; 
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_XTRAXPND) and self.CheckDone.trsp and self.Go.trsp then
					self.Speed.ex = 1.0; 
					self.Go.ex = not self.Go.ex; 
					self.Done.ex = true;  
					self.exSave = not self.exSave; 
				end; 
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_LVLCHK) and self.CheckDone.trsp and self.Go.trsp then
					self.levelCheck = not self.levelCheck; 
				end;
				if InputBinding.isPressed(InputBinding.AmazoneUX5200_LVLDWN) and not self.levelCheck and self.CheckDone.trsp and self.Go.trsp then
					self.Go.hubmast = true; 
					self.Done.hubmast = true;		
				elseif InputBinding.isPressed(InputBinding.AmazoneUX5200_LVLUP) and not self.levelCheck and self.CheckDone.trsp and self.Go.trsp then
					self.Go.hubmast = false; 
					self.Done.hubmast = true;
				elseif not self.Done.trsp and not self.levelCheck and self.CheckDone.trsp and self.Go.trsp then
					self.Done.hubmast = false; 
				end; 
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_ON) and self.CheckDone.trsp and self.fillLevel ~= 0 and self.Go.trsp then
					self.isTurnedOn = not self.isTurnedOn;
				end;
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_WRKLIGHT) and self.CheckDone.trsp and self.Go.trsp then
					self.workLightsActive = not self.workLightsActive;
				end;
				if InputBinding.hasEvent(InputBinding.AmazoneUX5200_DRVEMRK) then
					self.markOn = not self.markOn;
				end;				
			end;
		end;	
	end;	
end;

function AmazoneUX5200:updateTick(dt)

	if self.Go.pillar ~= nil and self.Done.pillar ~= false then
		self:anim("pillar", false); 
	end;	
	if self.Go.trsp ~= nil and self.Done.trsp ~= false then
		self:anim("trsp", false); 
	end;
	self:anim("hubmast", false); 
	if self.Go.ex ~= nil and self.Done.ex ~= false then
		self:anim("ex", false); 
	end;	
	if self:getIsActive() then
        if self.isTurnedOn then
			for k,sprayValve in pairs(self.sprayValves) do
                Utils.setEmittingState(sprayValve.particleSystems, self.isTurnedOn);
			end;
			if self.CheckDone.ex and self.Go.ex and not self.Done.ex then
				for l,exSprayValve in pairs(self.exSprayValves) do
					Utils.setEmittingState(exSprayValve.particleSystems, self.isTurnedOn);
				end;
			else
				for l,exSprayValve in pairs(self.exSprayValves) do
				Utils.setEmittingState(exSprayValve.particleSystems, false);
				end;
			end;
            self.lastSprayingArea = 0;   
            if self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31 then
                self.speedViolationTimer = self.speedViolationTimer - dt;
            else
                self.speedViolationTimer = self.speedViolationMaxTime;
            end;   
            if self.isServer then
                if self.speedViolationTimer > 0 then
                    local litersPerSecond = self.sprayLitersPerSecond[self.currentFillType];
                    if litersPerSecond == nil then
                        litersPerSecond = self.defaultSprayLitersPerSecond;
                    end
                    local usage = litersPerSecond * dt*0.001;   
                    local hasSpray = false;   
                    if self.capacity == 0 or self:getIsHired() then
                        hasSpray = true;
                        local sprayType = AmazoneUX5200.fillTypeToSprayType[self.currentFillType];
                        if sprayType ~= nil then
                            local sprayTypeDesc = AmazoneUX5200.sprayTypeIndexToDesc[sprayType];
                            local delta = usage*sprayTypeDesc.pricePerLiter
                            g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + delta;
                            g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + delta;
   
                            g_currentMission:addSharedMoney(-delta);
   
                               -- TODO update spray stats
                               --[[g_currentMission.missionStats.sprayUsageTotal = g_currentMission.missionStats.sprayUsageTotal + usage;
                               g_currentMission.missionStats.sprayUsageSession = g_currentMission.missionStats.sprayUsageSession + usage;]]
                        end;
                    else
                        if self.fillLevel > 0 then
                            hasSpray = true;
                            self:setFillLevel(self.fillLevel - usage, self.currentFillType);
                        end;
                    end;   
                    if hasSpray then
                        local cuttingAreasSend = {};
						if self.CheckDone.ex and self.Go.ex and not self.Done.ex then
							if self:getIsAreaActive(cuttingArea) then
								local x,y,z = getWorldTranslation(self.cuttingAreas[2].start);
								local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[2].width);
								local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[2].height);
								local sqm = math.abs((z1-z)*(x2-x) - (x1-x)*(z2-z)); -- this is the cross product with y=0
								self.lastSprayingArea = self.lastSprayingArea + sqm;
								table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
							end;
						else	
							if self:getIsAreaActive(cuttingArea) then
								local x,y,z = getWorldTranslation(self.cuttingAreas[1].start);
								local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[1].width);
								local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[1].height);
								local sqm = math.abs((z1-z)*(x2-x) - (x1-x)*(z2-z)); -- this is the cross product with y=0
								self.lastSprayingArea = self.lastSprayingArea + sqm;
								table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
							end;
						end;	
                        if (table.getn(cuttingAreasSend) > 0) then
                            SprayerAreaEvent.runLocally(cuttingAreasSend);
                            g_server:broadcastEvent(SprayerAreaEvent:new(cuttingAreasSend));
                        end;
                    end;
                end;
            end;
   
            if self.isClient then
                if not self.spraySoundEnabled and self:getIsActiveForSound() then
                    playSample(self.spraySound, 0, self.spraySoundVolume, 0);
                    setSamplePitch(self.spraySound, self.spraySoundPitchOffset);
                    self.spraySoundEnabled = true;
                end;
				if self.Done.trsp then
					if not self.hydraulicSoundEnabled and self:getIsActiveForSound() then
						playSample(self.hydraulicSound, 0, self.hydraulicSoundVolume, 0);
						setSamplePitch(self.hydraulicSound, self.hydraulicSoundPitchOffset);
						self.hydraulicSoundEnabled = true;
					end;
				else
					if self.hydraulicSoundEnabled then
						stopSample(self.hydraulicSound);
						self.hydraulicSoundEnabled = false;
					end;
				end; 
            end;   
            if self.fillLevel <= 0 and self.capacity ~= 0 then
                self:setIsTurnedOn(false, true);
            end;			
        else
            self.speedViolationTimer = self.speedViolationMaxTime;
			self:onDeactivateSounds()
			for k,sprayValve in pairs(self.sprayValves) do
                Utils.setEmittingState(sprayValve.particleSystems, false);
			end;
			for l,exSprayValve in pairs(self.exSprayValves) do
				Utils.setEmittingState(exSprayValve.particleSystems, false);
			end;			
        end;		
		if self.levelCheck then	
				local x,y,z = getWorldTranslation(self.groundCheck.right);
				local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
				local gCr = y-terrainHeight; 
				local x,y,z = getWorldTranslation(self.groundCheck.left);
				local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
				local gCl = y-terrainHeight; 
				local gC = math.floor(math.min(gCl,gCr) * 10) / 10;
				if (gC < 2.0) then 
					self.Go.hubmast = false; 
					self.Done.hubmast = true;
				else
					self.Go.hubmast = true; 
					self.Done.hubmast = true;
				end;
		end;
		for k, worklight in pairs(self.workLights) do
			setVisibility(worklight, self.workLightsActive);
		end;
		if not self.CheckDone.trsp and not self.Go.trsp then
			self.workLightsActive = false;
		end;		
		if self.markOn then		
			for k, markAreas in pairs(self.markAreas) do
				local x,y,z = getWorldTranslation(markAreas.point);
				local w = markAreas.width;
				local h = markAreas.height;
				local x1 = x + h/2
				local x2 = x - h/2
				local z1 = z + w/2
				local z2 = z - w/2			
				Utils.cutFruitArea( 1, x1, z2, x1, z1, x2, z2);
				Utils.cutFruitArea( 2, x1, z2, x1, z1, x2, z2);
				Utils.cutFruitArea( 3, x1, z2, x1, z1, x2, z2);
				Utils.cutFruitArea( 4, x1, z2, x1, z1, x2, z2);
				Utils.updateMeadowArea(x1, z2, x1, z1, x2, z2);
				Utils.updateCuttedMeadowArea(x1, z2, x1, z1, x2, z2);
			end;
		end; 		
    end;
	if self.isSprayerFilling and self.isServer then
		local disableFilling = false;
		if self:allowFillType(self.sprayerFillingFillType, false) then
			local oldFillLevel = self.fillLevel;   
			local delta = self.fillLitersPerSecond*dt*0.001;   
			local silo = g_currentMission:getSiloAmount(self.sprayerFillingFillType);
			if self.sprayerFillingIsSiloTrigger then
				if silo <= 0 then
					disableFilling = true;
				end;
				delta = math.min(delta, silo);
			end;  
			self:setFillLevel(self.fillLevel + delta, self.sprayerFillingFillType, true);
			local delta = self.fillLevel - oldFillLevel;
			if delta > 0 then
				if self.sprayerFillingIsSiloTrigger then
					g_currentMission:setSiloAmount(self.sprayerFillingFillType, silo - delta);
				else
					local sprayType = AmazoneUX5200.fillTypeToSprayType[self.sprayerFillingFillType];
					if sprayType ~= nil then
						local sprayTypeDesc = AmazoneUX5200.sprayTypeIndexToDesc[sprayType]
   							local price = delta*sprayTypeDesc.pricePerLiter;
						g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + price;
						g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + price;
						g_currentMission:addSharedMoney(-price);
					end;
				end;
			elseif self.fillLevel == self.capacity then
				disableFilling = true;
			end;
		else
			disableFilling = true;
		end;
		if disableFilling then
			self:setIsSprayerFilling(false);
		end;
	end;
end;

function AmazoneUX5200:draw()
	if self.isClient then
        if self.fillLevel <= 0 and self.capacity ~= 0 then
            g_currentMission:addExtraPrintText(g_i18n:getText("FirstFillTheTool"));
        end;   
        if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
            g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
        end;
		if not self.UX5200helpPanel then
			g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneUX5200_HLPANLON"), InputBinding.AmazoneUX5200_HLPANL);
		else	
			g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneUX5200_HLPANLOFF"), InputBinding.AmazoneUX5200_HLPANL);
			
			renderText(0.78, 0.86, 0.022, "- Amazone UX5200 Info -");
			if self.Go.trsp then
            renderText(0.7543, 0.83, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_TRSPRT)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.81, 0.02, g_i18n:getText("AmazoneUX5200_2"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.83, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_TRSPRT)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.81, 0.02, g_i18n:getText("AmazoneUX5200_1"));
			setTextColor(1, 1, 1, 1)
			end;
			if self.levelCheck then
			renderText(0.7543, 0.78, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_LVLCHK)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.76, 0.02, g_i18n:getText("AmazoneUX5200_6"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.78, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_LVLCHK)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.76, 0.02, g_i18n:getText("AmazoneUX5200_7"));
			setTextColor(1, 1, 1, 1)
			end;
			if self.isTurnedOn then
			renderText(0.7543, 0.73, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_ON)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.71, 0.02, g_i18n:getText("AmazoneUX5200_3"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.73, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_ON)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.71, 0.02, g_i18n:getText("AmazoneUX5200_4"));
			setTextColor(1, 1, 1, 1)
			end;
			if self.Go.ex then
			renderText(0.7543, 0.68, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_XTRAXPND)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.66, 0.02, g_i18n:getText("AmazoneUX5200_8"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.68, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_XTRAXPND)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.66, 0.02, g_i18n:getText("AmazoneUX5200_9"));
			setTextColor(1, 1, 1, 1)
			end;
			if self.markOn then
			renderText(0.7543, 0.63, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_DRVEMRK)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.61, 0.02, g_i18n:getText("AmazoneUX5200_12"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.63, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_DRVEMRK)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.61, 0.02, g_i18n:getText("AmazoneUX5200_13"));
			setTextColor(1, 1, 1, 1)
			end;
			if self.workLightsActive then
			renderText(0.7543, 0.58, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_WRKLIGHT)..": ");
			setTextColor(0, 1, 0, 1)
			renderText(0.78, 0.56, 0.02, g_i18n:getText("AmazoneUX5200_10"));
			setTextColor(1, 1, 1, 1)
			else
			renderText(0.7543, 0.58, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_WRKLIGHT)..": ");
			setTextColor(1, 0, 0, 1)
			renderText(0.78, 0.56, 0.02, g_i18n:getText("AmazoneUX5200_11"));
			setTextColor(1, 1, 1, 1)
			end;
			renderText(0.7543, 0.53, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_LVLUP)..": ");
			renderText(0.78, 0.51, 0.02, g_i18n:getText("AmazoneUX5200_5-1"));	
			renderText(0.7543, 0.48, 0.02, InputBinding.getKeyNamesOfDigitalAction(InputBinding.AmazoneUX5200_LVLDWN)..": ");
			renderText(0.78, 0.46, 0.02, g_i18n:getText("AmazoneUX5200_5-2"));
		end;
    end;
end;

function AmazoneUX5200:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	local loopCheck = self.loopCheck[varName];
	local speed = Utils.getNoNil(self.Speed[varName],1);
	local offset = 10;  
	local charId = getAnimCharacterSet(self.charId[varName]);
	local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
	if self.Done[varName] ~= false then
		if self.Go[varName] == true then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			setAnimTrackSpeedScale(charId, 0, speed);
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];	
			end;
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck then
				local setTime = getAnimClipDuration(charId, clipIndex) - getAnimClipDuration(charId, clipIndex);
				setAnimTrackTime(charId, 0, setTime);
			end;
		elseif self.Go[varName] == false then
			if loopCheck == true then
				self.Done[varName] = false;	
			else
				assignAnimTrackClip(charId , 0, clipIndex);
				setAnimTrackLoopState(charId, 0, loopCheck);
				setAnimTrackSpeedScale(charId, 0, -speed);
				enableAnimTrack(charId, 0);
				if getAnimTrackTime(charId, 0) <= 0 then
					self.Done[varName] = false;	
					self.CheckDone[varName] = self.Go[varName];
				end;
			end;
		elseif self.Go[varName] ~= nil then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			if self.Go[varName] < 0.0 then
				self.Go[varName] = 0.0;
			elseif self.Go[varName] > getAnimClipDuration(charId, clipIndex) then
				self.Go[varName] = getAnimClipDuration(charId, clipIndex);
			end;
			if getAnimTrackTime(charId, 0) >= self.Go[varName] then
				setAnimTrackSpeedScale(charId, 0, -speed);
			else
				setAnimTrackSpeedScale(charId, 0, speed);
			end;
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) <= self.Go[varName] + offset and getAnimTrackTime(charId, 0) >= self.Go[varName] - offset then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];
			end;
		end;
	end;
	if self.Done[varName] == false then
		disableAnimTrack(charId, 0);
	end;
end; 

function AmazoneUX5200:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;

function AmazoneUX5200:onAttach()
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
end; 	

function AmazoneUX5200:onDetach()
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
	if self.deactivateOnDetach then
        AmazoneUX5200.onDeactivate(self);
    else
        AmazoneUX5200.onDeactivateSounds(self);
    end;
end;

function AmazoneUX5200:onLeave()
    if self.deactivateOnLeave then
        AmazoneUX5200.onDeactivate(self);
    else
        AmazoneUX5200.onDeactivateSounds(self);
    end;
end;

function AmazoneUX5200:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'trsp="'..tostring(self.CheckDone.trsp)..'" mark="'..tostring(self.markOn)..'" ex="'..tostring(self.exSave)..'"';
	--local attributes = 'trsp="'..tostring(self.CheckDone.trsp)..'" ex="'..tostring(self.exSave)..'"';
	return attributes, nil;
end;

function AmazoneUX5200:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local trsp = getXMLBool(xmlFile, key.."#trsp");
	if trsp and not resetVehicles then
		self.Go.trsp = not self.Go.trsp; 
		self.Done.trsp = true; 
		self.Speed.trsp = 100.0;
	end; 	
	local ex = getXMLBool(xmlFile, key.."#ex");
	if ex and not resetVehicles then
		self.exSave = true; 
		if trsp then
			self.Go.ex = not self.Go.ex; 
			self.Done.ex = true; 
			self.Speed.ex = 100.0;
		end; 
	end;
	local mark = getXMLBool(xmlFile, key.."#mark");
		if mark and not resetVehicles then
		self.markOn = true; 
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function AmazoneUX5200:onDeactivate()
    self.speedViolationTimer = self.speedViolationMaxTime;
    self:setIsTurnedOn(false, true)
    AmazoneUX5200.onDeactivateSounds(self);
end;
  
function AmazoneUX5200:onDeactivateSounds()
    if self.spraySoundEnabled then
		stopSample(self.spraySound);
        self.spraySoundEnabled = false;
    end;
end;
  
function AmazoneUX5200:getIsTurnedOnAllowed(isTurnedOn)
    if not isTurnedOn or self.fillLevel > 0 or self.capacity == 0 then
        return true;
    end;
end;
  
function AmazoneUX5200:setIsTurnedOn(isTurnedOn, noEventSend)
    if isTurnedOn ~= self.isTurnedOn then
		if self:getIsTurnedOnAllowed(isTurnedOn) then
            SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
   
            self.isTurnedOn = isTurnedOn;
            if self.isClient then
				for k,sprayValve in pairs(self.sprayValves) do
                    Utils.setEmittingState(sprayValve.particleSystems, self.isTurnedOn);
				end;
				if self.CheckDone.ex and self.Go.ex and not self.Done.ex then
					for l,exSprayValve in pairs(self.exSprayValves) do
                    Utils.setEmittingState(exSprayValve.particleSystems, self.isTurnedOn);
					end;				
				end;  
                if not self.isTurnedOn and self.spraySoundEnabled then
                    stopSample(self.spraySound);
                    self.spraySoundEnabled = false;
                end;
            end;
            self.speedViolationTimer = self.speedViolationMaxTime;
        end;
    end;
end;

function AmazoneUX5200:setIsSprayerFilling(isFilling, fillType, isSiloTrigger, noEventSend)
		SprayerSetIsFillingEvent.sendEvent(self, isFilling, fillType, isSiloTrigger, noEventSend)
		if self.isSprayerFilling ~= isFilling then
			self.isSprayerFilling = isFilling;
			self.sprayerFillingFillType = fillType;
			self.sprayerFillingIsSiloTrigger = isSiloTrigger;
		end;
end;

 function AmazoneUX5200:addSprayerFillTrigger(trigger)
		if table.getn(self.sprayerFillTriggers) == 0 then
			g_currentMission:addActivatableObject(self.sprayerFillActivatable);
		end;
		table.insert(self.sprayerFillTriggers, trigger);
end;
   
function AmazoneUX5200:removeSprayerFillTrigger(trigger)
		for i=1, table.getn(self.sprayerFillTriggers) do
			if self.sprayerFillTriggers[i] == trigger then
				table.remove(self.sprayerFillTriggers, i);
				break;
			end;
		end;
		if table.getn(self.sprayerFillTriggers) == 0 then
			if self.isServer then
				self:setIsSprayerFilling(false);
			end;
			g_currentMission:removeActivatableObject(self.sprayerFillActivatable);
		end;
end;
   
   SprayerFillActivatable = {}
   local SprayerFillActivatable_mt = Class(SprayerFillActivatable);
   
function SprayerFillActivatable:new(AmazoneUX5200)
       local self = {};
       setmetatable(self, SprayerFillActivatable_mt);
   
       self.AmazoneUX5200 = AmazoneUX5200;
       self.activateText = "unknown";
   
       self.currentTrigger = nil;
   
       return self;
end;
  
   
function SprayerFillActivatable:getIsActivatable()
		self.currentTrigger = nil;
		if not self.AmazoneUX5200:getIsActiveForInput() or self.AmazoneUX5200.fillLevel == self.AmazoneUX5200.capacity then
			return false;
		end;
		-- find the first trigger which is activable
		for i=1, table.getn(self.AmazoneUX5200.sprayerFillTriggers) do
			local trigger = self.AmazoneUX5200.sprayerFillTriggers[i];
			if trigger:getIsActivatable(self.AmazoneUX5200) then
				self.currentTrigger = trigger;
				self:updateActivateText();
				return true;

			end;
       end;
       return false;
end;
   
function SprayerFillActivatable:onActivateObject()
		self.AmazoneUX5200:setIsSprayerFilling(not self.AmazoneUX5200.isSprayerFilling, self.currentTrigger.fillType, self.currentTrigger.isSiloTrigger);
		self:updateActivateText();
		g_currentMission:addActivatableObject(self);
end;
   
function SprayerFillActivatable:drawActivate()
       -- TODO draw icon
end;
   
function SprayerFillActivatable:updateActivateText()
		if self.AmazoneUX5200.isSprayerFilling then
			self.activateText = string.format(g_i18n:getText("stop_refill_OBJECT"), self.AmazoneUX5200.typeDesc);
		else
			self.activateText = string.format(g_i18n:getText("refill_OBJECT"), self.AmazoneUX5200.typeDesc);
		end;
end;